<?php

/************************************************************************/
/* DOCEBO CORE - Framework												*/
/* ============================================							*/
/*																		*/
/* Copyright (c) 2004													*/
/* http://www.docebo.com												*/
/*																		*/
/* This program is free software. You can redistribute it and/or modify	*/
/* it under the terms of the GNU General Public License as published by	*/
/* the Free Software Foundation; either version 2 of the License.		*/
/************************************************************************/

/**
 * @package 	DoceboCore
 * @category 	Image and multimedia managment
 *
 * @author Giovanni Derks <virtualdarkness[AT]gmail-com>
 * @version 	$Id: lib.multimedia.php 185 2006-03-30 14:40:18Z fabio $
 */

/**
 * @param 	string 	$src_path 	is the path of the folder of the uploaded image
 * @param 	string 	$dst_path 	is the path of the folder of the generated thumbinal
 * @param 	string 	$fn 		is the name of the image file
 * @param 	int 	$width 		is the width of the generated thumbinal
 * @param 	int 	$height 	is the height of the generated thumbinal
 * @param 	bool 	$forcesize 	if true then force the size to the one specified also if it is smaller
 */
function createPreview($src_path, $dst_path, $fn, $width, $height, $forcesize=true) {

	require_once($GLOBALS['where_framework'].'/lib/lib.upload.php');


	if(!extension_loaded('gd')) {
		return -1; // no gd
	}

	// The file
	$filename = $fn;
	$ext = strtolower(end(explode(".", $filename)));
	if(function_exists("getimagesize")) {

		// Get new dimensions
		list($width_orig, $height_orig) = getimagesize($src_path.$filename);

		if(($forcesize) || ($width_orig > $width) || ($height_orig > $height)) {

			if ($width && ($width_orig < $height_orig)) {
				$width = ($height / $height_orig) * $width_orig;
			} else {
				$height = ($width / $width_orig) * $height_orig;
			}
			// Resample
			$image = NULL;
			$image_p = imagecreatetruecolor($width, $height);
			switch($ext) {
				case "jpeg" :
				case "jpg" : {
					
					$image = imagecreatefromjpeg($src_path.$filename);
				};break;
				case "png" : {
					
					$image = imagecreatefrompng($src_path.$filename);
				};break;
				case "gif" : {
					
					if(!function_exists("imagecreatefromgif")) return -2;
					$image = imagecreatefromgif($src_path.$filename);
				};break;
				default : {
					// unknow format
					return -3;
				};break;
			}
			if($image == NULL) return -2; // can't open the image
			imagecopyresampled($image_p, $image, 0, 0, 0, 0, $width, $height, $width_orig, $height_orig);

		} else {
			// If there is no need to resize it..

			// If the destination is the same of the source we don't need to do anything..
			if ($src_path.$filename == $dst_path.$filename)
				return 1;
			// TODO:  ...
			/*else {
			} */
		}

		// Output
		$outname = $dst_path.$fn;
		sl_unlink($outname);

		if ( (($ext == "jpg") || ($ext == "jpeg")) && (function_exists("imagejpeg"))) {
			return imagejpeg($image_p, $outname, 90);
		}
		else if (($ext == "png") && (function_exists("imagejpeg"))) {
			$color = ImageColorAt($image_p, 1, 1);
			imagecolortransparent($image_p, $color);
			return imagepng($image_p, $outname);
		}
		else if (($ext == "gif") && (function_exists("imagegif")) && (function_exists("imagetruecolortopalette"))) {
			$image_gif = imagetruecolortopalette($image_p, true, 256);
			return imagegif($image_p, $outname);
		}

	}

}

/**
 * create a resized image from a tmp file
 * @param string 	$tmp_pathfile 	the location of tmp file
 * @param string 	$dst_pathfile 	the destination file
 * @param string 	$original_name 	the original name of the uploaded file
 * @param int 		$width 			the width to be forced
 * @param int 		$height 		the height to be forced
 * @param bool 		$forcesize 		if true resize the image
 * 
 * @return int	an error is a negative int, if 0 all is ok
 *			-1 ther isn't a required function, or the entire module 
 *			-2 general error
 *			-3 image type not suported
 *			-4 impossibile resize and the width or height exceed the param
 */
function createImageFromTmp($tmp_pathfile, $dst_pathfile, $original_name, $width, $height, $forcesize=true) {
	
	require_once($GLOBALS['where_framework'].'/lib/lib.upload.php');
	
	if(!extension_loaded('gd')) return -1;
	
	$dst_pathfile_rel = $GLOBALS['where_files_relative'].$dst_pathfile;
	$file_extension = strtolower(end(explode(".", $original_name)));
	
	if($forcesize === true) {
		
		// find image information
		if(!function_exists("getimagesize")) return -1;
		if(!function_exists("imagecopyresampled")) return -1;
		$img_info = @getimagesize($tmp_pathfile);
		if($img_info === false) return -2;
		
		$tmp_width 		= $img_info[0];
		$tmp_heigth 	= $img_info[1];
		
		// need resize
		if(($tmp_width > $width) || ($tmp_heigth > $height)) {
			
			// Resample
			if ($width && ($tmp_width < $tmp_heigth)) {
				$width = ($height / $tmp_heigth) * $tmp_width;
			} else {
				$height = ($width / $tmp_width) * $tmp_heigth;
			}
			
			$image 		= NULL;
			$image_p 	= imagecreatetruecolor($width, $height);
			switch($file_extension) {
				
				case "jpeg" :
				case "jpg" : {
					
					if(!function_exists("imagecreatefromjpeg")) return -4;
					$image = imagecreatefromjpeg($tmp_pathfile);
					
					
				};break;
				case "png" : {
					if(!function_exists("imagecreatefrompng")) return -4;
					$image = imagecreatefrompng($tmp_pathfile);
					
				};break;
				case "gif" : {
					
					if(!function_exists("imagecreatefromgif")) return -4;
					$image = imagecreatefromgif($tmp_pathfile);
				};break;
				default: {
					return -3;
				}
			}
			
			if($image == NULL) return -2; // can't open the image
			if(!imagecopyresampled($image_p, $image, 0, 0, 0, 0, $width, $height, $tmp_width, $tmp_heigth)) return -4;
			sl_unlink($dst_pathfile);
			
			switch($file_extension) {
				
				case "jpeg" :
				case "jpg" : {
					
					if(!function_exists("imagejpeg")) return -4;
					
					if(imagejpeg($image_p, $dst_pathfile_rel, 90)) return 0;
					else return -1;
				};break;
				case "png" : {
					
					if(!function_exists("ImageColorAt")) return -4;
					if(!function_exists("imagecolortransparent")) return -4;
					if(!function_exists("imagepng")) return -4;
					
					$color = ImageColorAt($image_p, 1, 1);
					imagecolortransparent($image_p, $color);
					if(imagepng($image_p, $dst_pathfile_rel) > 0) return 0;
					else return -1;
				};break;
				case "gif" : {
					
					if(!function_exists("imagegif")) return -4;
					if(!function_exists("imagetruecolortopalette")) return -4;
					
					$image_gif = imagetruecolortopalette($image_p, true, 256);
					if(imagegif($image_p, $dst_pathfile_rel) > 0) return 0;
					else return -1;
				};break;
			}
		}
	}
	// no resize needed
	sl_unlink($dst_pathfile);
	if(sl_upload($tmp_pathfile, $dst_pathfile)) return 0;
	return -2;
}

?>