<?php

/************************************************************************/
/* DOCEBO CORE - Framework						*/
/* ============================================				*/
/*									*/
/* Copyright (c) 2004							*/
/* http://www.docebo.com						*/
/*									*/
/* This program is free software. You can redistribute it and/or modify	*/
/* it under the terms of the GNU General Public License as published by	*/
/* the Free Software Foundation; either version 2 of the License.	*/
/************************************************************************/

if(!defined('IN_DOCEBO')) die('You cannot access this file directly');


/**
 * Acl user class
 * This class is for manage user login, preferences, etc
 * It store acl's security tockens in user session
 * For a detailed check use DoceboACL
 * To manage ACLs we must use DoceboACLManager
 *
 * @package  DoceboCore
 * @version  $Id: lib.user.php 419 2006-06-01 16:36:00Z fabio $
 * @category ACL
 * @uses 	 UserPreference
 * @author   Emanuele Sandri <esandri@tiscali.it>
 */

require_once( dirname(__FILE__) .'/lib.acl.php' );

require_once($GLOBALS['where_framework'].'/lib/lib.preference.php');

define("REFRESH_LAST_ENTER", 600);	//refresh the user last action every specified seconds

class DoceboUser {
	var $sprefix = '';
	var $acl = NULL;
	var $userid;
	var $idst;
	var $arrst = array();
	var $preference;

	/**
	 * create a DoceboACLUtil for given user
	 * and load all ST stored in session
	 **/
	function DoceboUser($userid, $sprefix) {
		$this->userid = $userid;
		$this->sprefix = $sprefix;

		$this->acl = new DoceboACL();
		$this->aclManager =& $this->acl->getACLManager();

		if( isset( $_SESSION[$sprefix.'_stlist'] ) ) {
			$this->arrst = unserialize( stripslashes( $_SESSION[$sprefix.'_stlist'] ) );
		}
		if( isset( $_SESSION[$sprefix.'_idst'] ) ) {
			$this->idst = $_SESSION[$sprefix.'_idst'];
		} else {
			$this->idst = $this->acl->getUserST( $userid );
		}

		$this->preference = new UserPreferences($this->idst);
	}

	function SaveInSession() {
		$_SESSION[$this->sprefix.'_idst'] = $this->idst;
		$_SESSION[$this->sprefix.'_username'] = $this->userid;
		$_SESSION[$this->sprefix.'_stlist'] = addslashes( serialize( $this->arrst ) );
		$_SESSION[$this->sprefix.'_log_ip'] = $_SERVER['REMOTE_ADDR'];
	}

	function isAnonymous()	{ return (strcmp($this->userid,'/Anonymous') == 0); }
	function isLoggedIn() 	{ return (strcmp($this->userid,'/Anonymous') != 0); }
	function getLogIp() 	{ return $_SESSION[$this->sprefix.'_log_ip']; }
	function getIdSt()		{ return $this->idst; }
	function getArrSt()		{ return $this->arrst; }
	function getUserId()	{ return $this->userid; }

	/**
	 * static function for load user from session
	 * @param string $prefix optional prefix for session variables
	 * @return mixed DoceboUser instance of logged in user if found user in session
	 *				 FALSE otherwise
	 **/
	function &createDoceboUserFromSession($prefix = 'base') {
		if( isset( $_SESSION[$prefix.'_username'] ) ) {
			$du = new DoceboUser( $_SESSION[$prefix.'_username'], $prefix );

			if(isset($_SESSION['user_enter_mark'])) {

				if($_SESSION['user_enter_mark'] < (time() - REFRESH_LAST_ENTER)) {
					$du->setLastEnter(date("Y-m-d H:i:s"));
					$_SESSION['user_enter_mark'] = time();
				}
			} else {
				$du->setLastEnter(date("Y-m-d H:i:s"));
				$_SESSION['user_enter_mark'] = time();
			}

			return $du;
		} else {
			$du = new DoceboUser( '/Anonymous', $prefix );
			return $du;
		}
	}

	/**
	 * static function for load user from login e password
	 * @param string $login login of the user
	 * @param string $password password of the user in clear text
	 * @param string $prefix optional prefix for session variables
	 * @return mixed DoceboUser instance of logged in user if success in login
	 *				 FALSE otherwise
	 **/
	function &createDoceboUserFromLogin($login, $password, $prefix = 'base', $new_lang = false) {
		
		if($login == '') {
			$false_var = FALSE;
			return $false_var;
		}
		
		$user_manager = new DoceboACLManager();
		$user_info = $user_manager->getUser(false, $login);

		$ret_value = false;
		if( $user_info === false )
			return $ret_value;

		if( $user_info[ACL_INFO_VALID] != '1' )
			return $ret_value;

		if($GLOBALS['framework']['ldap_used'] == 'on') {
			
			if($password == '') {
				$false_var = FALSE;
				return $false_var;
			}
			if($GLOBALS['framework']['ldap_port'] == '') 
				$GLOBALS['framework']['ldap_port'] = '389';

			//connect to ldap server
			if( !($ldap_conn = @ldap_connect( $GLOBALS['framework']['ldap_server'], $GLOBALS['framework']['ldap_port'] )) ) {
				die("Could not connect to ldap server");
			}

			//bind on server
			$ldap_user = ereg_replace( '\$user', $login, $GLOBALS['framework']['ldap_user_string'] );
			if (!(@ldap_bind($ldap_conn, $ldap_user, $password))) {
				ldap_close($ldap_conn);

				// Edited by Claudio Redaelli
				if ($GLOBALS['framework']['ldap_alternate_check'] == 'on') {
					if($user_info[ACL_INFO_PASS] != $user_manager->encrypt($password))
						return $ret_value;
				} else {
					$false_var = FALSE;
					return $false_var;
				}
				// End edit
			}
			ldap_close($ldap_conn);
		} elseif($user_info[ACL_INFO_PASS] != $user_manager->encrypt($password)) {
			return $ret_value;
		}
		unset($_SESSION[$prefix."_idst"]);
		$du = new DoceboUser( $login, $prefix );

		$du->setLastEnter(date("Y-m-d H:i:s"));
		$_SESSION['user_enter_mark'] = time();

		if($new_lang != false) {

			$du->preference->setLanguage($new_lang);
			$_SESSION['custom_lang'] = $new_lang;
		} else $_SESSION['custom_lang'] = $du->preference->getLanguage();

		if(function_exists('session_regenerate_id')) session_regenerate_id();
		return $du;
	}

	function setLastEnter($lastenter) {

		return $this->aclManager->updateUser($this->idst,
			FALSE,  FALSE,  FALSE, FALSE,  FALSE,  FALSE,  FALSE, FALSE,
			$lastenter );
	}

	/**
	 * This method load all security tokens associated to a section (course),
	 * test the match with user and save in user session positive ST
	 * @param string $section the section to load
	 **/
	function loadUserSectionST($section) {
		$this->arrst = $this->acl->getUserAllST($this->userid);
	}

	/**
	 * @return true if the user password is elapsed
	 */
	function isPasswordElapsed() {

		if($GLOBALS['framework']['ldap_used'] == 'on') return false;

		if($GLOBALS['framework']['pass_max_time_valid'] == 0) return false;

		$user_data = $this->aclManager->getUser($this->idst, false);
		if($user_data[ACL_INFO_PWD_EXPIRE_AT] == '0000-00-00 00:00:00')
			return false;

		$pwd_expire = fromDatetimeToTimestamp($user_data[ACL_INFO_PWD_EXPIRE_AT]);
		if(time() > $pwd_expire) return true;
		return false;
	}


	/**
	 * This method load all security tokens associated to a section (course),
	 * test the match with user and save in user session positive ST
	 * @param string $section the section to load
	 **/
	function saveUserSectionSTInSession($section) {
		$sprefix=$this->sprefix;

		if (!isset($_SESSION[$sprefix."_stlist"])) {
			$this->loadUserSectionST($section);
			$this->SaveInSession();
		}
	}

	/**
	 * This method test if in user's loaded ST ther'is a given ST
	 * @param int $st the security token to test
	 * @return bool TRUE, FALSE
	 **/
	function matchUserST($st) {
		return in_array( $st, $this->arrst );
	}

	/**
	 * This method test if user has a role
	 * @param string $roleid the role to test
	 * @return bool TRUE, FALSE
	 **/
	function matchUserRole($roleid) {

		return $this->matchUserST($this->acl->getRoleST($roleid));
	}

	/**
	 * This method test if user has one of given roles
	 * @param array $roles the array of roles to test
	 * @return bool TRUE, FALSE
	 **/
	function matchUserRoles($roles) {
		foreach( $roles as $r )
			if( $this->matchUserST($this->acl->getRoleST($r)) )
				return TRUE;
		return FALSE;
	}

	/**
	 * This method test if user has all passed roles
	 * @param array $roles the array of roles to test
	 * @return bool TRUE, FALSE
	 **/
	function matchUserAllRoles($roles) {
		foreach( $roles as $r )
			if( !$this->matchUserST($this->acl->getRoleST($r)) )
				return FALSE;
		return TRUE;
	}

	/**
	 * Get refernce to DoceboACL
	 * @return DoceboACL the DoceboACL object
	 **/
	function &getACL() {
		return $this->acl;
	}

	/**
	 * Get refernce to DoceboACLManager
	 * @return DoceboACLManager the DoceboACLManager object
	 **/
	function &getACLManager() {
		return $this->acl->getACLManager();
	}

	function getUserLevelId() {
 		$aclManager =& $this->acl->getACLManager();
		$arr_levels_id = $aclManager->getAdminLevels();
		$arr_levels_idst = array_values($arr_levels_id);
		$arr_levels_id = array_flip( $arr_levels_id );

		$arr_groups = $this->aclManager->getGroupsContainer($this->idst);

		$arr_user_level = array_intersect( $arr_levels_idst, $arr_groups );
		$arr_user_level = array_values($arr_user_level);
		if( count( $arr_user_level ) > 0 ) {
			$userlevel = $arr_user_level[0];
			return $arr_levels_id[$userlevel];
		}
		else if (!$GLOBALS["current_user"]->isAnonymous()) // NOT anonymous
			return $arr_levels_id[ADMIN_GROUP_USER];
		else // anonymous
			return FALSE;
	}

	function getUserName() {

		$user_info = $this->aclManager->getUser(getLogUserId(), false);
		return ( $user_info[ACL_INFO_LASTNAME].$user_info[ACL_INFO_FIRSTNAME]
			? $user_info[ACL_INFO_LASTNAME].' '.$user_info[ACL_INFO_FIRSTNAME]
			: $this->aclManager->relativeId($user_info[ACL_INFO_USERID]) );
	}
}


function getLogUserId() {

	return $GLOBALS['current_user']->getIdSt();
}

?>